@@ -23,6 +23,7 @@
 #include "misc.h"
 #include "stacking.h"
 #include "place.h"
+#include "client.h"
 #include "geom.h"
 #include "moveresize.h"
 #include "render/render.h"
@@ -38,6 +39,7 @@ struct _ObAppSettings
     GPatternSpec *class;
     GPatternSpec *name;
     GPatternSpec *role;
+    ObClientType  type;
 
     GravityPoint position;
     gboolean pos_given;
@@ -187,14 +189,13 @@ extern GSList *config_menu_files;
 extern GSList *config_per_app_settings;
 
 void config_startup(struct _ObParseInst *i);
-void config_shutdown();
+void config_shutdown(void);
 
 /*! Create an ObAppSettings structure with the default values */
-ObAppSettings* config_create_app_settings();
+ObAppSettings* config_create_app_settings(void);
 /*! Copies any settings in src to dest, if they are their default value in
   src. */
 void config_app_settings_copy_non_defaults(const ObAppSettings *src,
                                            ObAppSettings *dest);
 
-
 #endif
