@@ -21,6 +21,7 @@
 #include "client.h"
 #include "menu.h"
 #include "screen.h"
+#include "prop.h"
 #include "actions.h"
 #include "grab.h"
 #include "openbox.h"
@@ -29,7 +30,6 @@
 #include "render/theme.h"
 
 #define PADDING 2
-#define SEPARATOR_HEIGHT 3
 #define MAX_MENU_WIDTH 400
 
 #define ITEM_HEIGHT (ob_rr_theme->menu_font_height + 2*PADDING)
@@ -40,6 +40,9 @@
                          ButtonPressMask | ButtonReleaseMask)
 
 GList *menu_frame_visible;
+GHashTable *menu_frame_map;
+
+static RrAppearance *a_sep;
 
 static ObMenuEntryFrame* menu_entry_frame_new(ObMenuEntry *entry,
                                               ObMenuFrame *frame);
@@ -56,10 +59,18 @@ static Window createWindow(Window parent, gulong mask,
                          RrVisual(ob_rr_inst), mask, attrib);
 }
 
-GHashTable *menu_frame_map;
-
 void menu_frame_startup(gboolean reconfig)
 {
+    gint i;
+
+    a_sep = RrAppearanceCopy(ob_rr_theme->a_clear);
+    RrAppearanceAddTextures(a_sep, ob_rr_theme->menu_sep_width);
+    for (i = 0; i < ob_rr_theme->menu_sep_width; ++i) {
+        a_sep->texture[i].type = RR_TEXTURE_LINE_ART;
+        a_sep->texture[i].data.lineart.color =
+            ob_rr_theme->menu_sep_color;
+    }
+
     if (reconfig) return;
 
     menu_frame_map = g_hash_table_new(g_int_hash, g_int_equal);
@@ -67,6 +78,8 @@ void menu_frame_startup(gboolean reconfig)
 
 void menu_frame_shutdown(gboolean reconfig)
 {
+    RrAppearanceFree(a_sep);
+
     if (reconfig) return;
 
     g_hash_table_destroy(menu_frame_map);
@@ -89,6 +102,10 @@ ObMenuFrame* menu_frame_new(ObMenu *menu, guint show_from, ObClient *client)
     self->window = createWindow(RootWindow(ob_display, ob_screen),
                                 CWEventMask, &attr);
 
+    /* make it a popup menu type window */
+    PROP_SET32(self->window, net_wm_window_type, atom,
+               prop_atoms.net_wm_window_type_popup_menu);
+
     XSetWindowBorderWidth(ob_display, self->window, ob_rr_theme->mbwidth);
     XSetWindowBorder(ob_display, self->window,
                      RrColorPixel(ob_rr_theme->menu_border_color));
@@ -249,23 +266,24 @@ static void menu_frame_place_topmenu(ObMenuFrame *self, gint *x, gint *y)
 
 static void menu_frame_place_submenu(ObMenuFrame *self, gint *x, gint *y)
 {
-    gint overlap;
+    gint overlapx, overlapy;
     gint bwidth;
 
-    overlap = ob_rr_theme->menu_overlap;
+    overlapx = ob_rr_theme->menu_overlap_x;
+    overlapy = ob_rr_theme->menu_overlap_y;
     bwidth = ob_rr_theme->mbwidth;
 
     if (self->direction_right)
         *x = self->parent->area.x + self->parent->area.width -
-            overlap - bwidth;
+            overlapx - bwidth;
     else
-        *x = self->parent->area.x - self->area.width + overlap + bwidth;
+        *x = self->parent->area.x - self->area.width + overlapx + bwidth;
 
     *y = self->parent->area.y + self->parent_entry->area.y;
     if (config_menu_middle)
         *y -= (self->area.height - (bwidth * 2) - ITEM_HEIGHT) / 2;
     else
-        *y += overlap;
+        *y += overlapy;
 }
 
 void menu_frame_move_on_screen(ObMenuFrame *self, gint x, gint y,
@@ -327,12 +345,14 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
             th = ob_rr_theme->menu_title_height;
         } else {
             item_a = ob_rr_theme->a_menu_normal;
-            th = SEPARATOR_HEIGHT + 2*PADDING;
+            th = ob_rr_theme->menu_sep_width +
+                2*ob_rr_theme->menu_sep_paddingy;
         }
         break;
     default:
         g_assert_not_reached();
     }
+
     RECT_SET_SIZE(self->area, self->frame->inner_w, th);
     XResizeWindow(ob_display, self->window,
                   self->area.width, self->area.height);
@@ -430,31 +450,37 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
                     ob_rr_theme->menu_title_height -
                     2*ob_rr_theme->paddingy);
         } else {
-            RrAppearance *clear;
+            gint i;
 
-            /* unlabeled separaator */
-            XMoveResizeWindow(ob_display, self->text, PADDING, PADDING,
-                              self->area.width - 2*PADDING, SEPARATOR_HEIGHT);
+            /* unlabeled separator */
+            XMoveResizeWindow(ob_display, self->text, 0, 0,
+                              self->area.width,
+                              ob_rr_theme->menu_sep_width +
+                              2*ob_rr_theme->menu_sep_paddingy);
 
-            clear = ob_rr_theme->a_clear_tex;
-            clear->texture[0].type = RR_TEXTURE_LINE_ART;
-            clear->surface.parent = item_a;
-            clear->surface.parentx = PADDING;
-            clear->surface.parenty = PADDING;
-            clear->texture[0].data.lineart.color =
-                text_a->texture[0].data.text.color;
-            clear->texture[0].data.lineart.x1 = 2*PADDING;
-            clear->texture[0].data.lineart.y1 = SEPARATOR_HEIGHT/2;
-            clear->texture[0].data.lineart.x2 = self->area.width - 4*PADDING;
-            clear->texture[0].data.lineart.y2 = SEPARATOR_HEIGHT/2;
-            RrPaint(clear, self->text,
-                    self->area.width - 2*PADDING, SEPARATOR_HEIGHT);
+            a_sep->surface.parent = item_a;
+            a_sep->surface.parentx = 0;
+            a_sep->surface.parenty = 0;
+            for (i = 0; i < ob_rr_theme->menu_sep_width; ++i) {
+                a_sep->texture[i].data.lineart.x1 =
+                    ob_rr_theme->menu_sep_paddingx;
+                a_sep->texture[i].data.lineart.y1 =
+                    ob_rr_theme->menu_sep_paddingy + i;
+                a_sep->texture[i].data.lineart.x2 =
+                    self->area.width - ob_rr_theme->menu_sep_paddingx - 1;
+                a_sep->texture[i].data.lineart.y2 =
+                    ob_rr_theme->menu_sep_paddingy + i;
+            }
+
+            RrPaint(a_sep, self->text, self->area.width,
+                    ob_rr_theme->menu_sep_width +
+                    2*ob_rr_theme->menu_sep_paddingy);
         }
         break;
     }
 
     if (self->entry->type == OB_MENU_ENTRY_TYPE_NORMAL &&
-        self->entry->data.normal.icon_data)
+        self->entry->data.normal.icon)
     {
         RrAppearance *clear;
 
@@ -466,15 +492,12 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
                           - frame->item_margin.bottom);
 
         clear = ob_rr_theme->a_clear_tex;
-        clear->texture[0].type = RR_TEXTURE_RGBA;
-        clear->texture[0].data.rgba.width =
-            self->entry->data.normal.icon_width;
-        clear->texture[0].data.rgba.height =
-            self->entry->data.normal.icon_height;
-        clear->texture[0].data.rgba.alpha =
+        RrAppearanceClearTextures(clear);
+        clear->texture[0].type = RR_TEXTURE_IMAGE;
+        clear->texture[0].data.image.image =
+            self->entry->data.normal.icon;
+        clear->texture[0].data.image.alpha =
             self->entry->data.normal.icon_alpha;
-        clear->texture[0].data.rgba.data =
-            self->entry->data.normal.icon_data;
         clear->surface.parent = item_a;
         clear->surface.parentx = PADDING;
         clear->surface.parenty = frame->item_margin.top;
@@ -498,6 +521,7 @@ static void menu_entry_frame_render(ObMenuEntryFrame *self)
                           - frame->item_margin.bottom);
 
         clear = ob_rr_theme->a_clear_tex;
+        RrAppearanceClearTextures(clear);
         clear->texture[0].type = RR_TEXTURE_MASK;
         clear->texture[0].data.mask.mask =
             self->entry->data.normal.mask;
@@ -586,7 +610,8 @@ static gint menu_entry_frame_get_height(ObMenuEntryFrame *self,
             if (last_entry)
                 h -= ob_rr_theme->mbwidth;
         } else {
-            h += SEPARATOR_HEIGHT;
+            h += ob_rr_theme->menu_sep_width +
+                2*ob_rr_theme->menu_sep_paddingy - PADDING * 2;
         }
         break;
     }
@@ -671,7 +696,6 @@ void menu_frame_render(ObMenuFrame *self)
         XSetWindowBorder(ob_display, e->window,
                          RrColorPixel(ob_rr_theme->menu_border_color));
 
-
         text_a = (e->entry->type == OB_MENU_ENTRY_TYPE_NORMAL &&
                   !e->entry->data.normal.enabled ?
                   /* disabled */
@@ -689,7 +713,7 @@ void menu_frame_render(ObMenuFrame *self)
             tw = MIN(tw, MAX_MENU_WIDTH);
             th = ob_rr_theme->menu_font_height;
 
-            if (e->entry->data.normal.icon_data ||
+            if (e->entry->data.normal.icon ||
                 e->entry->data.normal.mask)
                 has_icon = TRUE;
             break;
@@ -700,7 +724,7 @@ void menu_frame_render(ObMenuFrame *self)
             tw = MIN(tw, MAX_MENU_WIDTH);
             th = ob_rr_theme->menu_font_height;
 
-            if (e->entry->data.normal.icon_data ||
+            if (e->entry->data.normal.icon ||
                 e->entry->data.normal.mask)
                 has_icon = TRUE;
 
@@ -717,7 +741,8 @@ void menu_frame_render(ObMenuFrame *self)
                     (ob_rr_theme->mbwidth - PADDING) *2;
             } else {
                 tw = 0;
-                th = SEPARATOR_HEIGHT;
+                th = ob_rr_theme->menu_sep_width +
+                    2*ob_rr_theme->menu_sep_paddingy - 2*PADDING;
             }
             break;
         }
@@ -1060,7 +1085,6 @@ void menu_frame_hide_all_client(ObClient *client)
     }
 }
 
-
 ObMenuFrame* menu_frame_under(gint x, gint y)
 {
     ObMenuFrame *ret = NULL;
